// deskbar - Deskbar info/control utility
//
// authors - in order of contribution
// jonas.sundstrom@kirilla.com
// 
// bugs & issues
// -- list feature doesn't work
// -- item info doesn't work
//
// libs: be, root
// 

#include <Deskbar.h>
#include <String.h>
#include <Entry.h>
#include <Mime.h>
#include <Roster.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

void print_usage (void);
	
int main (int32 argc, char ** argv)
{
	BDeskbar	deskbar;

	BString first_arg	=	argv[1];

	if (first_arg.IFindFirst("--count") == 0)
	{
		printf ("Deskbar items: %ld\n", deskbar.CountItems());
		return (0);
	}

	if (first_arg.IFindFirst("--list") == 0)
	{
		// since item_id's are not sequential, GetItemInfo()
		// isn't exactly great for retreiving the names of the
		// replicant items
		
		printf("not implemented\n");
		return (-1);
		
		/*		
		int32	deskbar_items	=	deskbar.CountItems();
		int32	found_items		=	0;
		int32	index			=	0;
		const char * item_name	=	NULL;
				
		while (found_items < deskbar_items)
		{
			//printf("index: %ld", index);
		
			if (deskbar.HasItem(index))
			{
				found_items++;
				
				deskbar.GetItemInfo(index, & item_name);
				printf("item %ld: %s\n", index, & item_name);
				free (& item_name);
			}
			
			index++;
		
			if (index > 4096)
			break;
		}
		
		if (found_items == deskbar_items)
			return (0);
		else
			return (-1);
		*/
	}	

	if (first_arg.IFindFirst("--hasitem=") == 0)
	{
		BString	name = first_arg;
		name.Remove(0, 10);
		
		if (deskbar.HasItem(name.String()) == B_OK)
		{
			printf("yes (always yes - API flawed?)\n");	// BUGGY API?
			return (0);
		}
		else
		{
			printf("no\n");
			return (-1);
		}

		// one could try to support
		// HasItem(int32 id) is useless since atoi(name.String() 
		// returns 0 if it can't convert, which is a valid id.. :(

		return (0);
	}

	if (first_arg.IFindFirst("--remove=") == 0)
	{
		BString	name = first_arg;
		name.Remove(0, 9);
		
		printf("%s\n", name.String());
		
		deskbar.RemoveItem(name.String());

		return (0);
	}

	if (first_arg.IFindFirst("--expanded") == 0)
	{
		if (deskbar.IsExpanded())
			printf("yes\n");
		else
			printf("no\n");

		return (0);
	}

	if (first_arg.IFindFirst("--setexpanded=") == 0)
	{
		BString	mode = first_arg;
		mode.Remove(0, 14);
		mode.ToLower();
		
		if (mode == "yes")
			deskbar.Expand(true);
		else if (mode == "no")
			deskbar.Expand(false);
		else
			print_usage();
		return (0);
	}

	if (first_arg.IFindFirst("--setlocation=") == 0)
	{
		BString	location = first_arg;
		location.Remove(0, 14);
		location.ToUpper();

		if (location == "B_DESKBAR_TOP")
			deskbar.SetLocation(B_DESKBAR_TOP, deskbar.IsExpanded());
		
		else if	(location == "B_DESKBAR_BOTTOM")
			deskbar.SetLocation(B_DESKBAR_BOTTOM, deskbar.IsExpanded());

		else if	(location == "B_DESKBAR_LEFT_BOTTOM")
			deskbar.SetLocation(B_DESKBAR_LEFT_BOTTOM, deskbar.IsExpanded());

		else if	(location == "B_DESKBAR_RIGHT_BOTTOM")
			deskbar.SetLocation(B_DESKBAR_RIGHT_BOTTOM, deskbar.IsExpanded());

		else if	(location == "B_DESKBAR_LEFT_TOP")
			deskbar.SetLocation(B_DESKBAR_LEFT_TOP, deskbar.IsExpanded());
			
		else if	(location == "B_DESKBAR_RIGHT_TOP")
			deskbar.SetLocation(B_DESKBAR_RIGHT_TOP, deskbar.IsExpanded());
			
		else
			printf("location type not recognised\n");

		return (0);
	}

	if (first_arg.IFindFirst("--location") == 0)
	{
		int32	location = deskbar.Location();

		if (location == B_DESKBAR_TOP)
			printf("B_DESKBAR_TOP\n");
		
		else if	(location == B_DESKBAR_BOTTOM)
			printf("B_DESKBAR_BOTTOM\n");

		else if	(location == B_DESKBAR_LEFT_BOTTOM)
			printf("B_DESKBAR_LEFT_BOTTOM\n");

		else if	(location == B_DESKBAR_RIGHT_BOTTOM)
			printf("B_DESKBAR_RIGHT_BOTTOM\n");

		else if	(location == B_DESKBAR_LEFT_TOP)
			printf("B_DESKBAR_LEFT_TOP\n");
			
		else if	(location == B_DESKBAR_RIGHT_TOP)
			printf("B_DESKBAR_RIGHT_TOP\n");
			
		else
			printf("unknown location type\n");

		return (0);
	}
	
	if (first_arg.IFindFirst("--frame") == 0)
	{
		BRect	frame	=	deskbar.Frame();

		printf("%ld\t%ld\t%ld\t%ld\n", int32(frame.left), int32(frame.top), int32(frame.right), int32(frame.bottom));

		return (0);
	}
	
	if (first_arg.IFindFirst("--add=") == 0)
	{
		BString	replicant_app = first_arg;
		replicant_app.Remove(0, 6);

		entry_ref 	app_entry_ref;
		
		if (BMimeType::IsValid(replicant_app.String()))
		{
			BRoster	roster;
			
			if (roster.FindApp(replicant_app.String(), & app_entry_ref) == B_OK)
				deskbar.AddItem(& app_entry_ref);
			else
				printf("%s\n", strerror(errno));
		}
		else
		{
			if (get_ref_for_path(replicant_app.String(), & app_entry_ref) == B_OK)
				deskbar.AddItem(& app_entry_ref);
			else
				printf("%s\n", strerror(errno));
		}

		return (0);
	}
	
	print_usage ();

	return (0);	
}

void print_usage (void)
{
	printf ("control:\n"
		" --add={app path or signature}\n"
		" --remove={name of replicant}\n"
		" --setlocation={B_DESKBAR_TOP|B_LEFT_BOTTOM|..}\n"
		" --setexpanded={yes|no}\n"
		"\n"
		"info:\n"
		" --list            (not implemented)\n"
		" --hasitem={name}  (does not seem to work)\n"
		" --count\n"
		" --frame\n"
		" --location\n"
		" --expanded\n");
}

